@@ -1,4 +1,5 @@
 #include "platform-funcs.hpp"
+#include "hotkey.hpp"
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -24,7 +25,11 @@
 #include <QStringList>
 #include <QRegularExpression>
 #include <QLibrary>
+#ifdef USE_PROCPS
 #include <proc/readproc.h>
+#else
+#include <libproc2/pids.h>
+#endif
 #include <fstream>
 #include <sstream>
 
@@ -35,18 +40,31 @@ typedef int (*keyPressFunc)(Display *, unsigned int, bool, unsigned long);
 static keyPressFunc pressFunc = nullptr;
 bool canSimulateKeyPresses = false;
 
+static QLibrary *libXssHandle = nullptr;
+typedef XScreenSaverInfo *(*XScreenSaverAllocInfoFunc)();
+typedef int (*XScreenSaverQueryInfoFunc)(Display *, Window, XScreenSaverInfo *);
+static XScreenSaverAllocInfoFunc allocSSFunc = nullptr;
+static XScreenSaverQueryInfoFunc querySSFunc = nullptr;
+bool canGetIdleTime = false;
+
+std::chrono::high_resolution_clock::time_point lastMouseLeftClickTime{};
+std::chrono::high_resolution_clock::time_point lastMouseMiddleClickTime{};
+std::chrono::high_resolution_clock::time_point lastMouseRightClickTime{};
+
 Display *disp()
 {
-	if (!xdisplay)
+	if (!xdisplay) {
 		xdisplay = XOpenDisplay(NULL);
+	}
 
 	return xdisplay;
 }
 
 void cleanupDisplay()
 {
-	if (!xdisplay)
+	if (!xdisplay) {
 		return;
+	}
 
 	XCloseDisplay(xdisplay);
 	xdisplay = 0;
@@ -55,22 +73,29 @@ void cleanupDisplay()
 static bool ewmhIsSupported()
 {
 	Display *display = disp();
+	if (!display) {
+		return false;
+	}
+	auto window = DefaultRootWindow(display);
+	if (!window) {
+		return false;
+	}
+
 	Atom netSupportingWmCheck =
 		XInternAtom(display, "_NET_SUPPORTING_WM_CHECK", true);
 	Atom actualType;
 	int format = 0;
 	unsigned long num = 0, bytes = 0;
 	unsigned char *data = NULL;
-	Window ewmh_window = 0;
+	Window ewmhWindow = 0;
 
-	int status = XGetWindowProperty(display, DefaultRootWindow(display),
-					netSupportingWmCheck, 0L, 1L, false,
-					XA_WINDOW, &actualType, &format, &num,
-					&bytes, &data);
+	int status = XGetWindowProperty(display, window, netSupportingWmCheck,
+					0L, 1L, false, XA_WINDOW, &actualType,
+					&format, &num, &bytes, &data);
 
 	if (status == Success) {
 		if (num > 0) {
-			ewmh_window = ((Window *)data)[0];
+			ewmhWindow = ((Window *)data)[0];
 		}
 		if (data) {
 			XFree(data);
@@ -78,29 +103,28 @@ static bool ewmhIsSupported()
 		}
 	}
 
-	if (ewmh_window) {
-		status = XGetWindowProperty(display, ewmh_window,
+	if (ewmhWindow) {
+		status = XGetWindowProperty(display, ewmhWindow,
 					    netSupportingWmCheck, 0L, 1L, false,
 					    XA_WINDOW, &actualType, &format,
 					    &num, &bytes, &data);
 		if (status != Success || num == 0 ||
-		    ewmh_window != ((Window *)data)[0]) {
-			ewmh_window = 0;
+		    ewmhWindow != ((Window *)data)[0]) {
+			ewmhWindow = 0;
 		}
 		if (status == Success && data) {
 			XFree(data);
 		}
 	}
-
-	return ewmh_window != 0;
+	return ewmhWindow != 0;
 }
 
 static QStringList getStates(Window window)
 {
 	QStringList states;
-
-	if (!ewmhIsSupported())
+	if (!window || !ewmhIsSupported()) {
 		return states;
+	}
 
 	Atom wmState = XInternAtom(disp(), "_NET_WM_STATE", true), type;
 	int format;
@@ -111,10 +135,13 @@ static QStringList getStates(Window window)
 					AnyPropertyType, &type, &format, &num,
 					&bytes, &data);
 
-	if (status == Success)
-		for (unsigned long i = 0; i < num; i++)
+	if (status == Success) {
+		for (unsigned long i = 0; i < num; i++) {
 			states.append(QString(
 				XGetAtomName(disp(), ((Atom *)data)[i])));
+		}
+		XFree(data);
+	}
 
 	return states;
 }
@@ -122,9 +149,6 @@ static QStringList getStates(Window window)
 static std::vector<Window> getTopLevelWindows()
 {
 	std::vector<Window> res;
-
-	res.resize(0);
-
 	if (!ewmhIsSupported()) {
 		return res;
 	}
@@ -136,10 +160,13 @@ static std::vector<Window> getTopLevelWindows()
 	Window *data = 0;
 
 	for (int i = 0; i < ScreenCount(disp()); ++i) {
-		Window rootWin = RootWindow(disp(), i);
+		Window rootWindow = RootWindow(disp(), i);
+		if (!rootWindow) {
+			continue;
+		}
 
-		int status = XGetWindowProperty(disp(), rootWin, netClList, 0L,
-						~0L, false, AnyPropertyType,
+		int status = XGetWindowProperty(disp(), rootWindow, netClList,
+						0L, ~0L, false, AnyPropertyType,
 						&actualType, &format, &num,
 						&bytes, (uint8_t **)&data);
 
@@ -147,8 +174,9 @@ static std::vector<Window> getTopLevelWindows()
 			continue;
 		}
 
-		for (unsigned long i = 0; i < num; ++i)
+		for (unsigned long i = 0; i < num; ++i) {
 			res.emplace_back(data[i]);
+		}
 
 		XFree(data);
 	}
@@ -156,24 +184,27 @@ static std::vector<Window> getTopLevelWindows()
 	return res;
 }
 
-static std::string GetWindowTitle(size_t i)
+std::string getWindowName(Window window)
 {
-	Window w = getTopLevelWindows().at(i);
+	auto display = disp();
+	if (!display || !window) {
+		return "";
+	}
 	std::string windowTitle;
 	char *name;
-
-	XTextProperty text;
-	int status = XGetTextProperty(
-		disp(), w, &text, XInternAtom(disp(), "_NET_WM_NAME", true));
-	if (status == 0)
-		status = XGetTextProperty(disp(), w, &text,
-					  XInternAtom(disp(), "WM_NAME", true));
-	name = reinterpret_cast<char *>(text.value);
-
-	if (status != 0 && name != nullptr) {
+	int status = XFetchName(display, window, &name);
+	if (status >= Success && name != nullptr) {
 		std::string str(name);
 		windowTitle = str;
 		XFree(name);
+	} else {
+		XTextProperty xtp_new_name;
+		if (XGetWMName(display, window, &xtp_new_name) != 0 &&
+		    xtp_new_name.value != nullptr) {
+			std::string str((const char *)xtp_new_name.value);
+			windowTitle = str;
+			XFree(xtp_new_name.value);
+		}
 	}
 
 	return windowTitle;
@@ -182,187 +213,124 @@ static std::string GetWindowTitle(size_t i)
 void GetWindowList(std::vector<std::string> &windows)
 {
 	windows.resize(0);
-
-	for (size_t i = 0; i < getTopLevelWindows().size(); ++i) {
-		if (GetWindowTitle(i) != "")
-			windows.emplace_back(GetWindowTitle(i));
+	for (auto window : getTopLevelWindows()) {
+		auto name = getWindowName(window);
+		if (name.empty()) {
+			continue;
+		}
+		windows.emplace_back(name);
 	}
 }
 
-// Overloaded
 void GetWindowList(QStringList &windows)
 {
 	windows.clear();
-
-	for (size_t i = 0; i < getTopLevelWindows().size(); ++i) {
-		if (GetWindowTitle(i) != "")
-			windows << QString::fromStdString(GetWindowTitle(i));
+	for (auto window : getTopLevelWindows()) {
+		auto name = getWindowName(window);
+		if (name.empty()) {
+			continue;
+		}
+		windows << QString::fromStdString(name);
 	}
 }
 
-void GetCurrentWindowTitle(std::string &title)
+int getActiveWindow(Window *&window)
 {
 	if (!ewmhIsSupported()) {
-		return;
+		return -1;
 	}
 
 	Atom active = XInternAtom(disp(), "_NET_ACTIVE_WINDOW", true);
 	Atom actualType;
 	int format;
 	unsigned long num, bytes;
+
+	auto rootWindow = DefaultRootWindow(disp());
+	if (!rootWindow) {
+		return -2;
+	}
+
+	return XGetWindowProperty(disp(), rootWindow, active, 0L, ~0L, false,
+				  AnyPropertyType, &actualType, &format, &num,
+				  &bytes, (uint8_t **)&window);
+}
+
+void GetCurrentWindowTitle(std::string &title)
+{
 	Window *data = 0;
-	char *name;
-
-	Window rootWin = RootWindow(disp(), 0);
-
-	int xstatus = XGetWindowProperty(disp(), rootWin, active, 0L, ~0L,
-					 false, AnyPropertyType, &actualType,
-					 &format, &num, &bytes,
-					 (uint8_t **)&data);
-
-	if (data == nullptr) {
+	if (getActiveWindow(data) != Success || !data) {
+		return;
+	}
+	if (!data[0]) {
+		XFree(data);
 		return;
 	}
 
-	int status = 0;
-	XTextProperty text;
-	if (xstatus == Success && data[0]) {
-		status = XGetTextProperty(disp(), data[0], &text,
-					  XInternAtom(disp(), "_NET_WM_NAME",
-						      true));
-		if (status == 0)
-			status = XGetTextProperty(disp(), data[0], &text,
-						  XInternAtom(disp(), "WM_NAME",
-							      true));
-	}
-	name = reinterpret_cast<char *>(text.value);
-
-	if (status != 0 && name != nullptr) {
-		std::string str(name);
-		title = str;
-		XFree(name);
+	auto name = getWindowName(data[0]);
+	XFree(data);
+	if (name.empty()) {
+		return;
 	}
+	title = name;
 }
 
-std::pair<int, int> getCursorPos()
+bool windowStatesAreSet(const std::string &windowTitle,
+			std::vector<QString> &expectedStates)
 {
-	std::pair<int, int> pos(0, 0);
-	Display *dpy;
-	Window root;
-	Window ret_root;
-	Window ret_child;
-	int root_x;
-	int root_y;
-	int win_x;
-	int win_y;
-	unsigned int mask;
-
-	dpy = disp();
-	root = XDefaultRootWindow(dpy);
-
-	if (XQueryPointer(dpy, root, &ret_root, &ret_child, &root_x, &root_y,
-			  &win_x, &win_y, &mask)) {
-		pos = std::pair<int, int>(root_x, root_y);
+	if (!ewmhIsSupported()) {
+		return false;
 	}
-	return pos;
+
+	std::vector<Window> windows = getTopLevelWindows();
+	for (auto &window : windows) {
+		auto name = getWindowName(window);
+		if (name.empty()) {
+			continue;
+		}
+
+		bool equals = windowTitle == name;
+		bool matches = QString::fromStdString(name).contains(
+			QRegularExpression(
+				QString::fromStdString(windowTitle)));
+
+		if (!(equals || matches)) {
+			continue;
+		}
+
+		QStringList states = getStates(window);
+		if (states.isEmpty()) {
+			if (expectedStates.empty()) {
+				return true;
+			}
+			return false;
+		}
+
+		for (const auto &state : expectedStates) {
+			if (!states.contains(state)) {
+				return false;
+			}
+		}
+		return true;
+	}
+	return false;
 }
 
 bool isMaximized(const std::string &title)
 {
-	if (!ewmhIsSupported())
-		return false;
-
-	// Find switch in top level windows
-	std::vector<Window> windows = getTopLevelWindows();
-	for (auto &window : windows) {
-		XTextProperty text;
-		int status = XGetTextProperty(
-			disp(), window, &text,
-			XInternAtom(disp(), "_NET_WM_NAME", true));
-		if (status == 0)
-			status = XGetTextProperty(disp(), window, &text,
-						  XInternAtom(disp(), "WM_NAME",
-							      true));
-		char *name = reinterpret_cast<char *>(text.value);
-
-		if (status == 0 || name == nullptr)
-			continue;
-
-		// True if switch equals window
-		bool equals = (title == name);
-		// True if switch matches window
-		bool matches = QString::fromStdString(name).contains(
-			QRegularExpression(QString::fromStdString(title)));
-
-		// If found, check if switch is maximized
-		if (equals || matches) {
-			QStringList states = getStates(window);
-
-			if (!states.isEmpty()) {
-				// True if window is maximized vertically
-				bool vertical = states.contains(
-					"_NET_WM_STATE_MAXIMIZED_VERT");
-				// True if window is maximized horizontally
-				bool horizontal = states.contains(
-					"_NET_WM_STATE_MAXIMIZED_HORZ");
-
-				return (vertical && horizontal);
-			}
-
-			break;
-		}
-	}
-
-	return false;
+	std::vector<QString> states;
+	states.emplace_back("_NET_WM_STATE_MAXIMIZED_VERT");
+	states.emplace_back("_NET_WM_STATE_MAXIMIZED_HORZ");
+	return windowStatesAreSet(title, states);
 }
 
 bool isFullscreen(const std::string &title)
 {
-	if (!ewmhIsSupported())
-		return false;
-
-	// Find switch in top level windows
-	std::vector<Window> windows = getTopLevelWindows();
-	for (auto &window : windows) {
-		XTextProperty text;
-		int status = XGetTextProperty(
-			disp(), window, &text,
-			XInternAtom(disp(), "_NET_WM_NAME", true));
-		if (status == 0)
-			status = XGetTextProperty(disp(), window, &text,
-						  XInternAtom(disp(), "WM_NAME",
-							      true));
-		char *name = reinterpret_cast<char *>(text.value);
-
-		if (status == 0 || name == nullptr)
-			continue;
-
-		// True if switch equals window
-		bool equals = (title == name);
-		// True if switch matches window
-		bool matches = QString::fromStdString(name).contains(
-			QRegularExpression(QString::fromStdString(title)));
-
-		// If found, check if switch is fullscreen
-		if (equals || matches) {
-			QStringList states = getStates(window);
-
-			if (!states.isEmpty()) {
-				// True if window is fullscreen
-				bool fullscreen = states.contains(
-					"_NET_WM_STATE_FULLSCREEN");
-
-				return (fullscreen);
-			}
-
-			break;
-		}
-	}
-
-	return false;
+	std::vector<QString> states;
+	states.emplace_back("_NET_WM_STATE_FULLSCREEN");
+	return windowStatesAreSet(title, states);
 }
 
-//exe switch is not quite what is expected but it works for now
+#ifdef USE_PROCPS
 void GetProcessList(QStringList &processes)
 {
 	processes.clear();
@@ -371,48 +339,56 @@ void GetProcessList(QStringList &processes)
 	memset(&proc_info, 0, sizeof(proc_info));
 	while (readproc(proc, &proc_info) != NULL) {
 		QString procName(proc_info.cmd);
-		if (!procName.isEmpty() && !processes.contains(proc_info.cmd)) {
-			processes << QString(proc_info.cmd);
+		if (!procName.isEmpty() && !processes.contains(procName)) {
+			processes << procName;
 		}
 	}
 	closeproc(proc);
 }
-
-int getForegroundProcessPid()
+#else
+void GetProcessList(QStringList &processes)
 {
-	if (!ewmhIsSupported()) {
-		return -1;
+	processes.clear();
+	struct pids_info *info = NULL;
+	struct pids_stack *stack;
+	enum pids_item Items[] = {
+		PIDS_CMD,
+	};
+
+	if (procps_pids_new(&info, Items, sizeof(Items) / sizeof(Items[0])) <
+	    0) {
+		return;
 	}
 
-	auto dpy = disp();
-	Atom active = XInternAtom(dpy, "_NET_ACTIVE_WINDOW", true);
-	Atom actualType;
-	int format;
-	unsigned long num, bytes;
-	Window *window = 0;
-	int pid = -1;
+	while ((stack = procps_pids_get(info, PIDS_FETCH_TASKS_ONLY))) {
+		auto cmd = PIDS_VAL(0, str, stack, info);
+		QString procName(cmd);
+		if (!procName.isEmpty() && !processes.contains(procName)) {
+			processes << procName;
+		}
+	}
+	procps_pids_unref(&info);
+}
+#endif
 
-	Window rootWin = RootWindow(dpy, 0);
-
-	int xstatus = XGetWindowProperty(dpy, rootWin, active, 0L, ~0L, false,
-					 AnyPropertyType, &actualType, &format,
-					 &num, &bytes, (uint8_t **)&window);
-
-	if (xstatus == 0 && window == nullptr) {
+long getForegroundProcessPid()
+{
+	Window *window;
+	if (getActiveWindow(window) != Success || !window || !*window) {
 		return -1;
 	}
-
 	Atom atom, actual_type;
 	int actual_format;
 	unsigned long nitems;
 	unsigned long bytes_after;
 	unsigned char *prop;
-	atom = XInternAtom(dpy, "_NET_WM_PID", True);
-	auto status = XGetWindowProperty(dpy, *window, atom, 0, 1024, False,
-					 AnyPropertyType, &actual_type,
+	long long pid;
+	atom = XInternAtom(disp(), "_NET_WM_PID", True);
+	auto status = XGetWindowProperty(disp(), *window, atom, 0, 1024, False,
+					 XA_CARDINAL, &actual_type,
 					 &actual_format, &nitems, &bytes_after,
 					 &prop);
-
+	XFree(window);
 	if (status != 0) {
 		return -2;
 	}
@@ -420,24 +396,42 @@ int getForegroundProcessPid()
 		return -3;
 	}
 
-	pid = prop[1] * 256;
-	pid += prop[0];
+	pid = *((long *)prop);
+	XFree(prop);
 	return pid;
 }
 
-std::string getProcNameFromPid(int pid)
+std::string getProcNameFromPid(long pid)
 {
 	std::string path = "/proc/" + std::to_string(pid) + "/comm";
 	std::ifstream t(path);
 	std::stringstream buffer;
 	buffer << t.rdbuf();
-	return buffer.str();
+	std::string name = buffer.str();
+	if (!name.empty() && name[name.length() - 1] == '\n') {
+		name.erase(name.length() - 1);
+	}
+	return name;
+}
+
+void GetForegroundProcessName(QString &proc)
+{
+	std::string temp;
+	GetForegroundProcessName(temp);
+	proc = QString::fromStdString(temp);
+}
+
+void GetForegroundProcessName(std::string &proc)
+{
+	proc.resize(0);
+	auto pid = getForegroundProcessPid();
+	proc = getProcNameFromPid(pid);
 }
 
 bool isInFocus(const QString &executable)
 {
-	auto pid = getForegroundProcessPid();
-	std::string current = getProcNameFromPid(pid);
+	std::string current;
+	GetForegroundProcessName(current);
 
 	// True if executable switch equals current window
 	bool equals = (executable.toStdString() == current);
@@ -450,22 +444,28 @@ bool isInFocus(const QString &executable)
 
 int secondsSinceLastInput()
 {
-	time_t idle_time;
-	static XScreenSaverInfo *mit_info;
-	Display *display;
-	int screen;
-
-	mit_info = XScreenSaverAllocInfo();
-
-	if ((display = disp()) == NULL) {
-		return -1;
+	if (!canGetIdleTime) {
+		return 0;
 	}
-	screen = DefaultScreen(display);
-	XScreenSaverQueryInfo(display, RootWindow(display, screen), mit_info);
+
+	auto display = disp();
+	if (!display) {
+		return 0;
+	}
+
+	auto window = DefaultRootWindow(display);
+	if (!window) {
+		return 0;
+	}
+
+	time_t idle_time;
+	auto mit_info = allocSSFunc();
+
+	auto status = querySSFunc(display, window, mit_info);
 	idle_time = (mit_info->idle) / 1000;
 	XFree(mit_info);
 
-	return idle_time;
+	return status != 0 ? idle_time : 0;
 }
 
 static std::unordered_map<HotkeyType, long> keyTable = {
@@ -595,8 +595,9 @@ void PressKeys(const std::vector<HotkeyType> keys, int duration)
 	}
 
 	Display *display = disp();
-	if (display == NULL)
+	if (!display) {
 		return;
+	}
 
 	// Press keys
 	for (auto &key : keys) {
@@ -626,17 +627,35 @@ void PressKeys(const std::vector<HotkeyType> keys, int duration)
 
 void PlatformInit()
 {
-	libXtstHandle = new QLibrary("libXtst.so", nullptr);
+	auto display = disp();
+	if (!display) {
+		return;
+	}
+
+	libXtstHandle = new QLibrary("libXtst", nullptr);
 	pressFunc = (keyPressFunc)libXtstHandle->resolve("XTestFakeKeyEvent");
 	int _;
 	canSimulateKeyPresses = pressFunc &&
-				!XQueryExtension(disp(), "XTEST", &_, &_, &_);
+				XQueryExtension(disp(), "XTEST", &_, &_, &_);
+
+	libXssHandle = new QLibrary("libXss", nullptr);
+	allocSSFunc = (XScreenSaverAllocInfoFunc)libXssHandle->resolve(
+		"XScreenSaverAllocInfo");
+	querySSFunc = (XScreenSaverQueryInfoFunc)libXssHandle->resolve(
+		"XScreenSaverQueryInfo");
+	canGetIdleTime = allocSSFunc && querySSFunc &&
+			 XQueryExtension(disp(), ScreenSaverName, &_, &_, &_);
 }
 
 void PlatformCleanup()
 {
-	delete libXtstHandle;
-	libXtstHandle = nullptr;
-
+	if (libXtstHandle) {
+		delete libXtstHandle;
+		libXtstHandle = nullptr;
+	}
+	if (libXssHandle) {
+		delete libXssHandle;
+		libXssHandle = nullptr;
+	}
 	cleanupDisplay();
 }
