@@ -4,23 +4,22 @@
 
 #include <QPushButton>
 #include <QListWidget>
-#include <unordered_map>
+#include <QCheckBox>
 
 class MacroActionRandom : public MultiMacroRefAction {
 public:
 	MacroActionRandom(Macro *m) : MacroAction(m), MultiMacroRefAction(m) {}
 	bool PerformAction();
-	void LogAction();
-	bool Save(obs_data_t *obj);
+	void LogAction() const;
+	bool Save(obs_data_t *obj) const;
 	bool Load(obs_data_t *obj);
-	std::string GetId() { return id; };
+	std::string GetId() const { return id; };
 	static std::shared_ptr<MacroAction> Create(Macro *m)
 	{
 		return std::make_shared<MacroActionRandom>(m);
 	}
 
-	// TODO: add weights to each macro ...
-	// std::unordered_map<MacroRef, int> _weights;
+	bool _allowRepeat = false;
 
 private:
 	MacroRef lastRandomMacro;
@@ -49,11 +48,15 @@ private slots:
 	void Add(const std::string &);
 	void Remove(int);
 	void Replace(int, const std::string &);
+	void AllowRepeatChanged(int);
 
 protected:
 	std::shared_ptr<MacroActionRandom> _entryData;
 
 private:
+	bool ShouldShowAllowRepeat();
+
 	MacroList *_list;
+	QCheckBox *_allowRepeat;
 	bool _loading = true;
 };
