@@ -10,44 +10,46 @@ bool MacroConditionMedia::_registered = MacroConditionFactory::Register(
 	{MacroConditionMedia::Create, MacroConditionMediaEdit::Create,
 	 "AdvSceneSwitcher.condition.media"});
 
-static std::map<MediaTimeRestriction, std::string> mediaTimeRestrictions = {
-	{MediaTimeRestriction::TIME_RESTRICTION_NONE,
+static std::map<MacroConditionMedia::Time, std::string> mediaTimeRestrictions = {
+	{MacroConditionMedia::Time::TIME_RESTRICTION_NONE,
 	 "AdvSceneSwitcher.mediaTab.timeRestriction.none"},
-	{MediaTimeRestriction::TIME_RESTRICTION_SHORTER,
+	{MacroConditionMedia::Time::TIME_RESTRICTION_SHORTER,
 	 "AdvSceneSwitcher.mediaTab.timeRestriction.shorter"},
-	{MediaTimeRestriction::TIME_RESTRICTION_LONGER,
+	{MacroConditionMedia::Time::TIME_RESTRICTION_LONGER,
 	 "AdvSceneSwitcher.mediaTab.timeRestriction.longer"},
-	{MediaTimeRestriction::TIME_RESTRICTION_REMAINING_SHORTER,
+	{MacroConditionMedia::Time::TIME_RESTRICTION_REMAINING_SHORTER,
 	 "AdvSceneSwitcher.mediaTab.timeRestriction.remainShorter"},
-	{MediaTimeRestriction::TIME_RESTRICTION_REMAINING_LONGER,
+	{MacroConditionMedia::Time::TIME_RESTRICTION_REMAINING_LONGER,
 	 "AdvSceneSwitcher.mediaTab.timeRestriction.remainLonger"},
 };
 
-static std::map<MediaState, std::string> mediaStates = {
-	{MediaState::OBS_MEDIA_STATE_NONE,
+static std::map<MacroConditionMedia::State, std::string> mediaStates = {
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_NONE,
 	 "AdvSceneSwitcher.mediaTab.states.none"},
-	{MediaState::OBS_MEDIA_STATE_PLAYING,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_PLAYING,
 	 "AdvSceneSwitcher.mediaTab.states.playing"},
-	{MediaState::OBS_MEDIA_STATE_OPENING,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_OPENING,
 	 "AdvSceneSwitcher.mediaTab.states.opening"},
-	{MediaState::OBS_MEDIA_STATE_BUFFERING,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_BUFFERING,
 	 "AdvSceneSwitcher.mediaTab.states.buffering"},
-	{MediaState::OBS_MEDIA_STATE_PAUSED,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_PAUSED,
 	 "AdvSceneSwitcher.mediaTab.states.paused"},
-	{MediaState::OBS_MEDIA_STATE_STOPPED,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_STOPPED,
 	 "AdvSceneSwitcher.mediaTab.states.stopped"},
-	{MediaState::OBS_MEDIA_STATE_ENDED,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_ENDED,
 	 "AdvSceneSwitcher.mediaTab.states.ended"},
-	{MediaState::OBS_MEDIA_STATE_ERROR,
+	{MacroConditionMedia::State::OBS_MEDIA_STATE_ERROR,
 	 "AdvSceneSwitcher.mediaTab.states.error"},
-	{MediaState::PLAYLIST_ENDED,
+	{MacroConditionMedia::State::PLAYLIST_ENDED,
 	 "AdvSceneSwitcher.mediaTab.states.playlistEnd"},
-	{MediaState::ANY, "AdvSceneSwitcher.mediaTab.states.any"},
+	{MacroConditionMedia::State::ANY,
+	 "AdvSceneSwitcher.mediaTab.states.any"},
 };
 
 MacroConditionMedia::~MacroConditionMedia()
 {
-	obs_source_t *mediasource = obs_weak_source_get_source(_source);
+	obs_source_t *mediasource =
+		obs_weak_source_get_source(_source.GetSource());
 	signal_handler_t *sh = obs_source_get_signal_handler(mediasource);
 	signal_handler_disconnect(sh, "media_stopped", MediaStopped, this);
 	signal_handler_disconnect(sh, "media_ended", MediaEnded, this);
@@ -57,7 +59,7 @@ MacroConditionMedia::~MacroConditionMedia()
 
 bool MacroConditionMedia::CheckTime()
 {
-	obs_source_t *s = obs_weak_source_get_source(_source);
+	obs_source_t *s = obs_weak_source_get_source(_source.GetSource());
 	auto duration = obs_source_media_get_duration(s);
 	auto currentTime = obs_source_media_get_time(s);
 	obs_source_release(s);
@@ -65,20 +67,20 @@ bool MacroConditionMedia::CheckTime()
 	bool match = false;
 
 	switch (_restriction) {
-	case MediaTimeRestriction::TIME_RESTRICTION_NONE:
+	case Time::TIME_RESTRICTION_NONE:
 		match = true;
 		break;
-	case MediaTimeRestriction::TIME_RESTRICTION_SHORTER:
+	case Time::TIME_RESTRICTION_SHORTER:
 		match = currentTime < _time.seconds * 1000;
 		break;
-	case MediaTimeRestriction::TIME_RESTRICTION_LONGER:
+	case Time::TIME_RESTRICTION_LONGER:
 		match = currentTime > _time.seconds * 1000;
 		break;
-	case MediaTimeRestriction::TIME_RESTRICTION_REMAINING_SHORTER:
+	case Time::TIME_RESTRICTION_REMAINING_SHORTER:
 		match = duration > currentTime &&
 			duration - currentTime < _time.seconds * 1000;
 		break;
-	case MediaTimeRestriction::TIME_RESTRICTION_REMAINING_LONGER:
+	case Time::TIME_RESTRICTION_REMAINING_LONGER:
 		match = duration > currentTime &&
 			duration - currentTime > _time.seconds * 1000;
 		break;
@@ -91,7 +93,7 @@ bool MacroConditionMedia::CheckTime()
 
 bool MacroConditionMedia::CheckState()
 {
-	obs_source_t *s = obs_weak_source_get_source(_source);
+	obs_source_t *s = obs_weak_source_get_source(_source.GetSource());
 	obs_media_state currentState = obs_source_media_get_state(s);
 	obs_source_release(s);
 
@@ -100,24 +102,24 @@ bool MacroConditionMedia::CheckState()
 	int expectedState = static_cast<int>(_state);
 
 	switch (_state) {
-	case MediaState::OBS_MEDIA_STATE_STOPPED:
+	case State::OBS_MEDIA_STATE_STOPPED:
 		match = _stopped || currentState == OBS_MEDIA_STATE_STOPPED;
 		break;
-	case MediaState::OBS_MEDIA_STATE_ENDED:
+	case State::OBS_MEDIA_STATE_ENDED:
 		match = _ended || currentState == OBS_MEDIA_STATE_ENDED;
 		break;
-	case MediaState::PLAYLIST_ENDED:
+	case State::PLAYLIST_ENDED:
 		match = CheckPlaylistEnd(currentState);
 		break;
-	case MediaState::ANY:
+	case State::ANY:
 		match = true;
 		break;
-	case MediaState::OBS_MEDIA_STATE_NONE:
-	case MediaState::OBS_MEDIA_STATE_PLAYING:
-	case MediaState::OBS_MEDIA_STATE_OPENING:
-	case MediaState::OBS_MEDIA_STATE_BUFFERING:
-	case MediaState::OBS_MEDIA_STATE_PAUSED:
-	case MediaState::OBS_MEDIA_STATE_ERROR:
+	case State::OBS_MEDIA_STATE_NONE:
+	case State::OBS_MEDIA_STATE_PLAYING:
+	case State::OBS_MEDIA_STATE_OPENING:
+	case State::OBS_MEDIA_STATE_BUFFERING:
+	case State::OBS_MEDIA_STATE_PAUSED:
+	case State::OBS_MEDIA_STATE_ERROR:
 		match = currentState == expectedState;
 		break;
 	default:
@@ -142,13 +144,13 @@ bool MacroConditionMedia::CheckPlaylistEnd(const obs_media_state currentState)
 
 bool MacroConditionMedia::CheckMediaMatch()
 {
-	if (!_source) {
+	if (!_source.GetSource()) {
 		return false;
 	}
 	bool match = false;
 	bool matched = CheckState() && CheckTime();
 
-	if (matched && !(_onlyMatchonChagne && _alreadyMatched)) {
+	if (matched && !(_onlyMatchOnChagne && _alreadyMatched)) {
 		match = true;
 	}
 	_alreadyMatched = matched;
@@ -161,42 +163,53 @@ bool MacroConditionMedia::CheckMediaMatch()
 	return match;
 }
 
+void MacroConditionMedia::HandleSceneChange()
+{
+	UpdateMediaSourcesOfSceneList();
+	_lastConfigureScene = switcher->currentScene;
+}
+
 bool MacroConditionMedia::CheckCondition()
 {
 	bool match = false;
 	switch (_sourceType) {
-	case MediaSourceType::ANY:
-		for (auto &source : _sources) {
+	case Type::ANY:
+		for (auto &source : _sourceGroup) {
 			match = match || source.CheckCondition();
 		}
 		break;
-	case MediaSourceType::ALL: {
+	case Type::ALL: {
 		bool res = true;
-		for (auto &source : _sources) {
+		for (auto &source : _sourceGroup) {
 			res = res && source.CheckCondition();
 		}
 		match = res;
 		break;
 	}
-	case MediaSourceType::SOURCE:
+	case Type::SOURCE:
 		match = CheckMediaMatch();
 		break;
 	default:
 		break;
 	}
+
+	if (_lastConfigureScene != switcher->currentScene) {
+		HandleSceneChange();
+	}
+
 	return match;
 }
 
-bool MacroConditionMedia::Save(obs_data_t *obj)
+bool MacroConditionMedia::Save(obs_data_t *obj) const
 {
 	MacroCondition::Save(obj);
-	obs_data_set_string(obj, "source", GetWeakSourceName(_source).c_str());
+	_source.Save(obj);
 	_scene.Save(obj);
 	obs_data_set_int(obj, "sourceType", static_cast<int>(_sourceType));
 	obs_data_set_int(obj, "state", static_cast<int>(_state));
 	obs_data_set_int(obj, "restriction", static_cast<int>(_restriction));
 	_time.Save(obj);
-	obs_data_set_bool(obj, "matchOnChagne", _onlyMatchonChagne);
+	obs_data_set_bool(obj, "matchOnChagne", _onlyMatchOnChagne);
 	obs_data_set_int(obj, "version", 0);
 	return true;
 }
@@ -222,41 +235,43 @@ static bool enumSceneItem(obs_scene_t *, obs_sceneitem_t *item, void *ptr)
 	return true;
 }
 
-void forMediaSourceOnSceneAddMediaCondition(
-	OBSWeakSource sceneWeakSource, MacroConditionMedia *origCond,
-	std::vector<MacroConditionMedia> &conditions)
+void MacroConditionMedia::UpdateMediaSourcesOfSceneList()
 {
-	conditions.clear();
+	_sourceGroup.clear();
+	if (!_scene.GetScene(false)) {
+		return;
+	}
 	std::vector<OBSWeakSource> mediaSources;
-	auto s = obs_weak_source_get_source(sceneWeakSource);
+	auto s = obs_weak_source_get_source(_scene.GetScene(false));
 	auto scene = obs_scene_from_source(s);
 	obs_scene_enum_items(scene, enumSceneItem, &mediaSources);
 	obs_source_release(s);
+	_sourceGroup.reserve(mediaSources.size());
 
 	for (auto &source : mediaSources) {
-		MacroConditionMedia cond(*origCond);
-		cond._sourceType = MediaSourceType::SOURCE;
-		cond._source = source;
-		conditions.push_back(cond);
+		MacroConditionMedia cond(*this);
+		cond._sourceType = Type::SOURCE;
+		cond._source.SetSource(source);
+		_sourceGroup.push_back(cond);
 	}
 }
 
 bool MacroConditionMedia::Load(obs_data_t *obj)
 {
 	MacroCondition::Load(obj);
-	const char *sourceName = obs_data_get_string(obj, "source");
-	_source = GetWeakSourceByName(sourceName);
+	_source.Load(obj);
 	_scene.Load(obj);
-	_sourceType = static_cast<MediaSourceType>(
-		obs_data_get_int(obj, "sourceType"));
-	_state = static_cast<MediaState>(obs_data_get_int(obj, "state"));
-	_restriction = static_cast<MediaTimeRestriction>(
+	_sourceType = static_cast<Type>(obs_data_get_int(obj, "sourceType"));
+	_state = static_cast<MacroConditionMedia::State>(
+		obs_data_get_int(obj, "state"));
+	_restriction = static_cast<MacroConditionMedia::Time>(
 		obs_data_get_int(obj, "restriction"));
 	_time.Load(obj);
-	_onlyMatchonChagne = obs_data_get_bool(obj, "matchOnChagne");
+	_onlyMatchOnChagne = obs_data_get_bool(obj, "matchOnChagne");
 
-	if (_sourceType == MediaSourceType::SOURCE) {
-		obs_source_t *mediasource = obs_weak_source_get_source(_source);
+	if (_sourceType == Type::SOURCE) {
+		obs_source_t *mediasource =
+			obs_weak_source_get_source(_source.GetSource());
 		signal_handler_t *sh =
 			obs_source_get_signal_handler(mediasource);
 		signal_handler_connect(sh, "media_stopped", MediaStopped, this);
@@ -265,33 +280,30 @@ bool MacroConditionMedia::Load(obs_data_t *obj)
 		obs_source_release(mediasource);
 	}
 
-	forMediaSourceOnSceneAddMediaCondition(_scene.GetScene(), this,
-					       _sources);
+	UpdateMediaSourcesOfSceneList();
 	if (!obs_data_has_user_value(obj, "version")) {
-		if (_state == MediaState::OBS_MEDIA_STATE_ENDED) {
-			_state = MediaState::PLAYLIST_ENDED;
+		if (_state ==
+		    MacroConditionMedia::State::OBS_MEDIA_STATE_ENDED) {
+			_state = MacroConditionMedia::State::PLAYLIST_ENDED;
 		}
 	}
 	return true;
 }
 
-std::string MacroConditionMedia::GetShortDesc()
+std::string MacroConditionMedia::GetShortDesc() const
 {
 	switch (_sourceType) {
-	case MediaSourceType::SOURCE:
-		if (_source) {
-			return GetWeakSourceName(_source);
-		}
-		break;
-	case MediaSourceType::ANY:
-		if (_scene.GetScene()) {
+	case Type::SOURCE:
+		return _source.ToString();
+	case Type::ANY:
+		if (_scene.GetScene(false)) {
 			return obs_module_text(
 				       "AdvSceneSwitcher.condition.media.anyOnScene") +
 			       std::string(" ") + _scene.ToString();
 		}
 		break;
-	case MediaSourceType::ALL:
-		if (_scene.GetScene()) {
+	case Type::ALL:
+		if (_scene.GetScene(false)) {
 			return obs_module_text(
 				       "AdvSceneSwitcher.condition.media.allOnScene") +
 			       std::string(" ") + _scene.ToString();
@@ -305,7 +317,8 @@ std::string MacroConditionMedia::GetShortDesc()
 
 void MacroConditionMedia::ClearSignalHandler()
 {
-	obs_source_t *mediasource = obs_weak_source_get_source(_source);
+	obs_source_t *mediasource =
+		obs_weak_source_get_source(_source.GetSource());
 	signal_handler_t *sh = obs_source_get_signal_handler(mediasource);
 	signal_handler_disconnect(sh, "media_stopped", MediaStopped, this);
 	signal_handler_disconnect(sh, "media_ended", MediaEnded, this);
@@ -315,7 +328,8 @@ void MacroConditionMedia::ClearSignalHandler()
 
 void MacroConditionMedia::ResetSignalHandler()
 {
-	obs_source_t *mediasource = obs_weak_source_get_source(_source);
+	obs_source_t *mediasource =
+		obs_weak_source_get_source(_source.GetSource());
 	signal_handler_t *sh = obs_source_get_signal_handler(mediasource);
 	signal_handler_disconnect(sh, "media_stopped", MediaStopped, this);
 	signal_handler_disconnect(sh, "media_ended", MediaEnded, this);
@@ -356,7 +370,7 @@ void MacroConditionMedia::MediaNext(void *data, calldata_t *)
 	media->_next = true;
 }
 
-static void populateMediaTimeRestrictions(QComboBox *list)
+static void populateMediaTimes(QComboBox *list)
 {
 	for (auto entry : mediaTimeRestrictions) {
 		list->addItem(obs_module_text(entry.second.c_str()));
@@ -370,34 +384,45 @@ static void populateMediaStates(QComboBox *list)
 	}
 }
 
-static void addAnyAndAllStates(QComboBox *list)
+static void populateSourceTypes(QComboBox *list)
 {
-	list->insertItem(
-		1,
-		obs_module_text("AdvSceneSwitcher.condition.media.anyOnScene"));
-	list->insertItem(
-		1,
-		obs_module_text("AdvSceneSwitcher.condition.media.allOnScene"));
+	list->clear();
+	list->addItem(
+		obs_module_text("AdvSceneSwitcher.condition.media.source"),
+		static_cast<int>(MacroConditionMedia::Type::SOURCE));
+	list->addItem(
+		obs_module_text("AdvSceneSwitcher.condition.media.anyOnScene"),
+		static_cast<int>(MacroConditionMedia::Type::ANY));
+	list->addItem(
+		obs_module_text("AdvSceneSwitcher.condition.media.allOnScene"),
+		static_cast<int>(MacroConditionMedia::Type::ALL));
 }
 
 MacroConditionMediaEdit::MacroConditionMediaEdit(
 	QWidget *parent, std::shared_ptr<MacroConditionMedia> entryData)
 	: QWidget(parent),
-	  _scenes(new SceneSelectionWidget(window())),
-	  _mediaSources(new QComboBox()),
+	  _sourceTypes(new QComboBox()),
+	  _scenes(new SceneSelectionWidget(window(), true, true, true, true,
+					   true)),
+	  _sources(new SourceSelectionWidget(this, QStringList(), true)),
 	  _states(new QComboBox()),
 	  _timeRestrictions(new QComboBox()),
 	  _time(new DurationSelection()),
 	  _onChange(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.condition.media.matchOnChange")))
-
 {
 	_states->setToolTip(obs_module_text(
 		"AdvSceneSwitcher.condition.media.inconsistencyInfo"));
 
-	QWidget::connect(_mediaSources,
-			 SIGNAL(currentTextChanged(const QString &)), this,
-			 SLOT(SourceChanged(const QString &)));
+	auto sources = GetMediaSourceNames();
+	sources.sort();
+	_sources->SetSourceNameList(sources);
+
+	QWidget::connect(_sourceTypes, SIGNAL(currentIndexChanged(int)), this,
+			 SLOT(SourceTypeChanged(int)));
+	QWidget::connect(_sources,
+			 SIGNAL(SourceChanged(const SourceSelection &)), this,
+			 SLOT(SourceChanged(const SourceSelection &)));
 	QWidget::connect(_scenes, SIGNAL(SceneChanged(const SceneSelection &)),
 			 this, SLOT(SceneChanged(const SceneSelection &)));
 	QWidget::connect(_states, SIGNAL(currentIndexChanged(int)), this,
@@ -411,14 +436,14 @@ MacroConditionMediaEdit::MacroConditionMediaEdit(
 	QWidget::connect(_onChange, SIGNAL(stateChanged(int)), this,
 			 SLOT(OnChangeChanged(int)));
 
-	populateMediaSelection(_mediaSources);
-	addAnyAndAllStates(_mediaSources);
+	populateSourceTypes(_sourceTypes);
 	populateMediaStates(_states);
-	populateMediaTimeRestrictions(_timeRestrictions);
+	populateMediaTimes(_timeRestrictions);
 
 	QHBoxLayout *entryLayout = new QHBoxLayout;
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
-		{"{{mediaSources}}", _mediaSources},
+		{"{{sourceTypes}}", _sourceTypes},
+		{"{{mediaSources}}", _sources},
 		{"{{scenes}}", _scenes},
 		{"{{states}}", _states},
 		{"{{timeRestrictions}}", _timeRestrictions},
@@ -436,28 +461,20 @@ MacroConditionMediaEdit::MacroConditionMediaEdit(
 	_loading = false;
 }
 
-void MacroConditionMediaEdit::SourceChanged(const QString &text)
+void MacroConditionMediaEdit::SourceTypeChanged(int idx)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
 	std::lock_guard<std::mutex> lock(switcher->m);
+	_entryData->_sourceType = static_cast<MacroConditionMedia::Type>(
+		_sourceTypes->itemData(idx).toInt());
 
-	if (text ==
-	    obs_module_text("AdvSceneSwitcher.condition.media.anyOnScene")) {
-		_entryData->_sourceType = MediaSourceType::ANY;
-	} else if (text ==
-		   obs_module_text(
-			   "AdvSceneSwitcher.condition.media.allOnScene")) {
-		_entryData->_sourceType = MediaSourceType::ALL;
-	} else {
-		_entryData->_sources.clear();
-		_entryData->_sourceType = MediaSourceType::SOURCE;
+	if (_entryData->_sourceType == MacroConditionMedia::Type::SOURCE) {
+		_entryData->_sourceGroup.clear();
 	}
 
-	_entryData->ClearSignalHandler();
-	_entryData->_source = GetWeakSourceByQString(text);
 	_entryData->ResetSignalHandler();
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -465,6 +482,23 @@ void MacroConditionMediaEdit::SourceChanged(const QString &text)
 	SetWidgetVisibility();
 }
 
+void MacroConditionMediaEdit::SourceChanged(const SourceSelection &source)
+{
+	if (_loading || !_entryData) {
+		return;
+	}
+
+	std::lock_guard<std::mutex> lock(switcher->m);
+	_entryData->_sourceGroup.clear();
+	_entryData->_sourceType = MacroConditionMedia::Type::SOURCE;
+	_entryData->ClearSignalHandler();
+	_entryData->_source = source;
+	_entryData->ResetSignalHandler();
+	emit HeaderInfoChanged(
+		QString::fromStdString(_entryData->GetShortDesc()));
+	SetWidgetVisibility();
+}
+
 void MacroConditionMediaEdit::SceneChanged(const SceneSelection &s)
 {
 	if (_loading || !_entryData) {
@@ -473,21 +507,21 @@ void MacroConditionMediaEdit::SceneChanged(const SceneSelection &s)
 
 	std::lock_guard<std::mutex> lock(switcher->m);
 	_entryData->_scene = s;
-	forMediaSourceOnSceneAddMediaCondition(_entryData->_scene.GetScene(),
-					       _entryData.get(),
-					       _entryData->_sources);
+	_entryData->UpdateMediaSourcesOfSceneList();
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
 }
 
-MediaState getMediaStateFromIdx(int idx)
+MacroConditionMedia::State getMediaStateFromIdx(int idx)
 {
-	if (idx < static_cast<int>(MediaState::LAST_OBS_MEDIA_STATE)) {
-		return static_cast<MediaState>(idx);
+	if (idx < static_cast<int>(
+			  MacroConditionMedia::State::LAST_OBS_MEDIA_STATE)) {
+		return static_cast<MacroConditionMedia::State>(idx);
 	} else {
-		return static_cast<MediaState>(
+		return static_cast<MacroConditionMedia::State>(
 			idx -
-			static_cast<int>(MediaState::LAST_OBS_MEDIA_STATE) +
+			static_cast<int>(
+				MacroConditionMedia::State::LAST_OBS_MEDIA_STATE) +
 			custom_media_states_offset);
 	}
 }
@@ -500,10 +534,8 @@ void MacroConditionMediaEdit::StateChanged(int index)
 
 	std::lock_guard<std::mutex> lock(switcher->m);
 	_entryData->_state = getMediaStateFromIdx(index);
-	if (_entryData->_sourceType != MediaSourceType::SOURCE) {
-		forMediaSourceOnSceneAddMediaCondition(
-			_entryData->_scene.GetScene(), _entryData.get(),
-			_entryData->_sources);
+	if (_entryData->_sourceType != MacroConditionMedia::Type::SOURCE) {
+		_entryData->UpdateMediaSourcesOfSceneList();
 	}
 }
 
@@ -513,19 +545,18 @@ void MacroConditionMediaEdit::TimeRestrictionChanged(int index)
 		return;
 	}
 
-	if (static_cast<MediaTimeRestriction>(index) ==
-	    MediaTimeRestriction::TIME_RESTRICTION_NONE) {
+	if (static_cast<MacroConditionMedia::Time>(index) ==
+	    MacroConditionMedia::Time::TIME_RESTRICTION_NONE) {
 		_time->setDisabled(true);
 	} else {
 		_time->setDisabled(false);
 	}
 
 	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_restriction = static_cast<MediaTimeRestriction>(index);
-	if (_entryData->_sourceType != MediaSourceType::SOURCE) {
-		forMediaSourceOnSceneAddMediaCondition(
-			_entryData->_scene.GetScene(), _entryData.get(),
-			_entryData->_sources);
+	_entryData->_restriction =
+		static_cast<MacroConditionMedia::Time>(index);
+	if (_entryData->_sourceType != MacroConditionMedia::Type::SOURCE) {
+		_entryData->UpdateMediaSourcesOfSceneList();
 	}
 }
 
@@ -537,10 +568,8 @@ void MacroConditionMediaEdit::TimeChanged(double seconds)
 
 	std::lock_guard<std::mutex> lock(switcher->m);
 	_entryData->_time.seconds = seconds;
-	if (_entryData->_sourceType != MediaSourceType::SOURCE) {
-		forMediaSourceOnSceneAddMediaCondition(
-			_entryData->_scene.GetScene(), _entryData.get(),
-			_entryData->_sources);
+	if (_entryData->_sourceType != MacroConditionMedia::Type::SOURCE) {
+		_entryData->UpdateMediaSourcesOfSceneList();
 	}
 }
 
@@ -552,10 +581,8 @@ void MacroConditionMediaEdit::TimeUnitChanged(DurationUnit unit)
 
 	std::lock_guard<std::mutex> lock(switcher->m);
 	_entryData->_time.displayUnit = unit;
-	if (_entryData->_sourceType != MediaSourceType::SOURCE) {
-		forMediaSourceOnSceneAddMediaCondition(
-			_entryData->_scene.GetScene(), _entryData.get(),
-			_entryData->_sources);
+	if (_entryData->_sourceType != MacroConditionMedia::Type::SOURCE) {
+		_entryData->UpdateMediaSourcesOfSceneList();
 	}
 }
 
@@ -566,29 +593,31 @@ void MacroConditionMediaEdit::OnChangeChanged(int value)
 	}
 
 	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_onlyMatchonChagne = value;
-	if (_entryData->_sourceType != MediaSourceType::SOURCE) {
-		forMediaSourceOnSceneAddMediaCondition(
-			_entryData->_scene.GetScene(), _entryData.get(),
-			_entryData->_sources);
+	_entryData->_onlyMatchOnChagne = value;
+	if (_entryData->_sourceType != MacroConditionMedia::Type::SOURCE) {
+		_entryData->UpdateMediaSourcesOfSceneList();
 	}
 }
 
 void MacroConditionMediaEdit::SetWidgetVisibility()
 {
-	_scenes->setVisible(_entryData->_sourceType != MediaSourceType::SOURCE);
+	_sources->setVisible(_entryData->_sourceType ==
+			     MacroConditionMedia::Type::SOURCE);
+	_scenes->setVisible(_entryData->_sourceType !=
+			    MacroConditionMedia::Type::SOURCE);
 	if (!_onChange->isChecked()) {
 		_onChange->hide();
 	}
 }
 
-int getIdxFromMediaState(MediaState state)
+int getIdxFromMediaState(MacroConditionMedia::State state)
 {
-	if (state < MediaState::LAST_OBS_MEDIA_STATE) {
+	if (state < MacroConditionMedia::State::LAST_OBS_MEDIA_STATE) {
 		return static_cast<int>(state);
 	} else {
 		return static_cast<int>(state) - custom_media_states_offset +
-		       static_cast<int>(MediaState::LAST_OBS_MEDIA_STATE);
+		       static_cast<int>(
+			       MacroConditionMedia::State::LAST_OBS_MEDIA_STATE);
 	}
 }
 
@@ -598,32 +627,18 @@ void MacroConditionMediaEdit::UpdateEntryData()
 		return;
 	}
 
-	switch (_entryData->_sourceType) {
-	case MediaSourceType::ANY:
-		_mediaSources->setCurrentText(obs_module_text(
-			"AdvSceneSwitcher.condition.media.anyOnScene"));
-		break;
-	case MediaSourceType::ALL:
-		_mediaSources->setCurrentText(obs_module_text(
-			"AdvSceneSwitcher.condition.media.allOnScene"));
-		break;
-	case MediaSourceType::SOURCE:
-		_mediaSources->setCurrentText(
-			GetWeakSourceName(_entryData->_source).c_str());
-		break;
-	default:
-		break;
-	}
-
+	_sourceTypes->setCurrentIndex(_sourceTypes->findData(
+		static_cast<int>(_entryData->_sourceType)));
+	_sources->SetSource(_entryData->_source);
 	_scenes->SetScene(_entryData->_scene);
 	_states->setCurrentIndex(getIdxFromMediaState(_entryData->_state));
 	_timeRestrictions->setCurrentIndex(
 		static_cast<int>(_entryData->_restriction));
 	_time->SetDuration(_entryData->_time);
 	if (_entryData->_restriction ==
-	    MediaTimeRestriction::TIME_RESTRICTION_NONE) {
+	    MacroConditionMedia::Time::TIME_RESTRICTION_NONE) {
 		_time->setDisabled(true);
 	}
-	_onChange->setChecked(_entryData->_onlyMatchonChagne);
+	_onChange->setChecked(_entryData->_onlyMatchOnChagne);
 	SetWidgetVisibility();
 }
