@@ -13,9 +13,9 @@ class MacroConditionReplayBuffer : public MacroCondition {
 public:
 	MacroConditionReplayBuffer(Macro *m) : MacroCondition(m) {}
 	bool CheckCondition();
-	bool Save(obs_data_t *obj);
+	bool Save(obs_data_t *obj) const;
 	bool Load(obs_data_t *obj);
-	std::string GetId() { return id; };
+	std::string GetId() const { return id; };
 	static std::shared_ptr<MacroCondition> Create(Macro *m)
 	{
 		return std::make_shared<MacroConditionReplayBuffer>(m);
